#YOLOv5 🚀 by Ultralytics, GPL-3.0 license
"""
Run inference on images, videos, directories, streams, etc.

Usage:
    $ python path/to/detect.py --source path/to/img.jpg --weights yolov5s.pt --img 640
"""

import argparse
import os
import sys
from pathlib import Path

import cv2
import numpy as np
import torch
import torch.backends.cudnn as cudnn

FILE = Path(__file__).resolve()
ROOT = FILE.parents[0]  # YOLOv5 root directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH
ROOT = Path(os.path.relpath(ROOT, Path.cwd()))  # relative

from models.experimental import attempt_load
from utils.datasets import LoadImages, LoadStreams
from utils.general import apply_classifier, check_img_size, check_imshow, check_requirements, check_suffix, colorstr, \
    increment_path, non_max_suppression, print_args, save_one_box, scale_coords, set_logging, \
    strip_optimizer, xyxy2xywh
from utils.plots import Annotator, colors
from utils.torch_utils import load_classifier, select_device, time_sync

import torch_mlu
import torch_mlu.core.mlu_quantize as mlu_quantize
import torch_mlu.core.mlu_model as ct

import numpy as np
import os

from models.yolo import Model
from torch_mlu.core.utils import dump_utils

# 边缘端MLU220
ct.set_core_version('MLU220')

def run(weights=ROOT / 'yolov5s.pt',  # model.pt path(s)
        source=ROOT / 'data/images',  # file/dir/URL/glob, 0 for webcam
        imgsz=640,  # inference size (pixels)
        conf_thres=0.25,  # confidence threshold
        iou_thres=0.45,  # NMS IOU threshold
        max_det=1000,  # maximum detections per image
        device='',  # cuda device, i.e. 0 or 0,1,2,3 or cpu
        view_img=False,  # show results
        save_txt=False,  # save results to *.txt
        save_conf=False,  # save confidences in --save-txt labels
        save_crop=False,  # save cropped prediction boxes
        nosave=False,  # do not save images/videos
        classes=None,  # filter by class: --class 0, or --class 0 2 3
        agnostic_nms=False,  # class-agnostic NMS
        augment=False,  # augmented inference
        visualize=False,  # visualize features
        update=False,  # update all models
        project=ROOT / 'runs/detect',  # save results to project/name
        name='exp',  # save results to project/name
        exist_ok=False,  # existing project/name ok, do not increment
        line_thickness=3,  # bounding box thickness (pixels)
        hide_labels=False,  # hide labels
        hide_conf=False,  # hide confidences
        half=False,  # use FP16 half-precision inference
        dnn=False,  # use OpenCV DNN for ONNX inference
        cfg="cpu",
        jit=False,
        offline_model="",
        ):
    source = str(source)
    save_img = not nosave and not source.endswith('.txt')  # save inference images
    webcam = source.isnumeric() or source.endswith('.txt') or source.lower().startswith(
        ('rtsp://', 'rtmp://', 'http://', 'https://'))

    # Directories
    save_dir = increment_path(Path(project) / name, exist_ok=exist_ok)  # increment run
    (save_dir / 'labels' if save_txt else save_dir).mkdir(parents=True, exist_ok=True)  # make dir

    # Initialize
    set_logging()
    device = select_device(device)
    half &= device.type != 'cpu'  # half precision only supported on CUDA

    stride, names = 64, [f'class{i}' for i in range(1000)]  # assign defaults

    imgsz = check_img_size(imgsz, s=stride)  # check image size


    dataset = LoadImages(source, img_size=imgsz, stride=stride, auto=False)
    bs = 1
    vid_path, vid_writer = [None] * bs, [None] * bs

    model = Model('./models/yolov5s_m_bird.yaml')
    model.float().fuse()

    quantized_net = torch_mlu.core.mlu_quantize.quantize_dynamic_mlu(model)

#    dump_utils.register_dump_hook(quantized_net)

    state_dict = torch.load("./unzip_2024_sm_best8_flock_s15_3733_900epochs_int16_qua.pt")
#    print("state_dict.keys():",state_dict.keys())

    quantized_net.load_state_dict(state_dict, strict=True)
    quantized_net.float().eval()
    dump_utils.register_dump_hook(quantized_net)
    quantized_net.to(ct.mlu_device())


    batchsize = 1

    for path, img0, im0s, vid_cap in dataset:
        print("img0.shape:",img0.shape)

#        print("im0s.shape:",im0s.shape)
    #    t1 = time_sync()
    #    img0 = img0[np.newaxis,:]
    #    img0 = np.repeat(img0,batchsize,axis=0)
    #    img = torch.from_numpy(img0)
    #    img = img.float()
#   #         img = torch.from_numpy(img).to(device)
#   #         img = img.half() if half else img.float()  # uint8 to fp16/32
    #    img /= 255.0
#   #     img = img / 255.0  # 0 - 255 to 0.0 - 1.0
    #    if device==torch.device("cpu"):
    #        img = img.float()
#   #    else:
#   #         img = img.half()
    #    if len(img.shape) == 3:
    #        img = img[None]  # expand for batch dim
    #    t2 = time_sync()
#   #     dt[0] += t2 - t1
       
        img = torch.from_numpy(img0)
        img = img.half() if half else img.float()
        img = img / 255.0 # firstconv = False use it
        if len(img.shape) == 3:
            img = img[None]  
 
        print("path:",path)
        print("img:",img.shape)
        print("im0s:",im0s.shape)

#        print("after_img:",img)

        pred = quantized_net(img.to(ct.mlu_device()))[0]

        dump_utils.save_data("output/", "mlu")
        
#        torch.save(quantized_model.state_dict(), 'unzip_2021_m_best_int8_qua.pt')
#        pred = non_max_suppression(pred.cpu(), conf_thres, iou_thres, classes, agnostic_nms, max_det=max_det)
#        print(pred)


        value = dump_utils.diff_data("output/dump_cpu_data.pth", "output/dump_mlu_data.pth",mode="COS")
        print("value:",value)
#    for i, det in enumerate(pred):  # per image
##        seen += 1
#        if webcam:  # batch_size >= 1
#            p, s, im0, frame = path[i], f'{i}: ', im0s[i].copy(), dataset.count
#        else:
#            p, s, im0, frame = path, '', im0s.copy(), getattr(dataset, 'frame', 0)
#
#        p = Path(p)  # to Path
#        save_path = str(save_dir / p.name)  # img.jpg
#        txt_path = str(save_dir / 'labels' / p.stem) + ('' if dataset.mode == 'image' else f'_{frame}')  # img.txt
#        s += '%gx%g ' % img.shape[2:]  # print string
#        gn = torch.tensor(im0.shape)[[1, 0, 1, 0]]  # normalization gain whwh
#        imc = im0.copy() if save_crop else im0  # for save_crop
#        annotator = Annotator(im0, line_width=line_thickness, example=str(names))
#        if len(det):
#            # Rescale boxes from img_size to im0 size
#            det[:, :4] = scale_coords(img.shape[2:], det[:, :4], im0.shape).round()
#
#            # Print results
#            for c in det[:, -1].unique():
#                n = (det[:, -1] == c).sum()  # detections per class
#                s += f"{n} {names[int(c)]}{'s' * (n > 1)}, "  # add to string
#                s += f"{n} {names[int(c)]}s, "  # add to string
#            # Write results
#            for *xyxy, conf, cls in reversed(det):
#                if save_txt:  # Write to file
#                    xywh = (xyxy2xywh(torch.tensor(xyxy).view(1, 4)) / gn).view(-1).tolist()  # normalized xywh
#                    line = (cls, *xywh, conf) if save_conf else (cls, *xywh)  # label format
#                    with open(txt_path + '.txt', 'a') as f:
#                        f.write(('%g ' * len(line)).rstrip() % line + '\n')
#
#                if save_img or save_crop or view_img:  # Add bbox to image
#                    c = int(cls)  # integer class
#                    label = None if hide_labels else (names[c] if hide_conf else f'{names[c]} {conf:.2f}')
#                    annotator.box_label(xyxy, label, color=colors(c, True))
#                    if save_crop:
#                        save_one_box(xyxy, imc, file=save_dir / 'crops' / names[c] / f'{p.stem}.jpg', BGR=True)
#
#        # Print time (inference-only)
#        print(f'{s}Done. ({t3 - t2:.3f}s)')
#
#        # Stream results
#        im0 = annotator.result()
#        if view_img:
#            cv2.imshow(str(p), im0)
#            cv2.waitKey(1)  # 1 millisecond
#
#        # Save results (image with detections)
#        if save_img:
#            if dataset.mode == 'image':
#                cv2.imwrite(save_path, im0)
#            else:  # 'video' or 'stream'
#                if vid_path[i] != save_path:  # new video
#                    vid_path[i] = save_path
#                    if isinstance(vid_writer[i], cv2.VideoWriter):
#                        vid_writer[i].release()  # release previous video writer
#                    if vid_cap:  # video
#                        fps = vid_cap.get(cv2.CAP_PROP_FPS)
#                        w = int(vid_cap.get(cv2.CAP_PROP_FRAME_WIDTH))
#                        h = int(vid_cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
#                    else:  # stream
#                        fps, w, h = 30, im0.shape[1], im0.shape[0]
#                        save_path += '.mp4'
#                    vid_writer[i] = cv2.VideoWriter(save_path, cv2.VideoWriter_fourcc(*'mp4v'), fps, (w, h))
#                vid_writer[i].write(im0)
#
#    # Print results
##    t = tuple(x / seen * 1E3 for x in dt)  # speeds per image
##    print(f'Speed: %.1fms pre-process, %.1fms inference, %.1fms NMS per image at shape {(1, 3, *imgsz)}' % t)
#    if save_txt or save_img:
#        s = f"\n{len(list(save_dir.glob('labels/*.txt')))} labels saved to {save_dir / 'labels'}" if save_txt else ''
#        print(f"Results saved to {colorstr('bold', save_dir)}{s}")
#    if update:
#        strip_optimizer(weights)  # update model (to fix SourceChangeWarning)




def parse_opt():
    parser = argparse.ArgumentParser()
    parser.add_argument('--weights', nargs='+', type=str, default=ROOT / 'yolov5s.pt', help='model path(s)')
    parser.add_argument('--source', type=str, default=ROOT / 'data/images_dump', help='file/dir/URL/glob, 0 for webcam')
    parser.add_argument('--imgsz', '--img', '--img-size', nargs='+', type=int, default=[640], help='inference size h,w')
    parser.add_argument('--conf-thres', type=float, default=0.25, help='confidence threshold')
    parser.add_argument('--iou-thres', type=float, default=0.45, help='NMS IoU threshold')
    parser.add_argument('--max-det', type=int, default=1000, help='maximum detections per image')
    parser.add_argument('--device', default='', help='cuda device, i.e. 0 or 0,1,2,3 or cpu')
    parser.add_argument('--view-img', action='store_true', help='show results')
    parser.add_argument('--save-txt', action='store_true', help='save results to *.txt')
    parser.add_argument('--save-conf', action='store_true', help='save confidences in --save-txt labels')
    parser.add_argument('--save-crop', action='store_true', help='save cropped prediction boxes')
    parser.add_argument('--nosave', action='store_true', help='do not save images/videos')
    parser.add_argument('--classes', nargs='+', type=int, help='filter by class: --classes 0, or --classes 0 2 3')
    parser.add_argument('--agnostic-nms', action='store_true', help='class-agnostic NMS')
    parser.add_argument('--augment', action='store_true', help='augmented inference')
    parser.add_argument('--visualize', action='store_true', help='visualize features')
    parser.add_argument('--update', action='store_true', help='update all models')
    parser.add_argument('--project', default=ROOT / 'runs/detect', help='save results to project/name')
    parser.add_argument('--name', default='exp', help='save results to project/name')
    parser.add_argument('--exist-ok', action='store_true', help='existing project/name ok, do not increment')
    parser.add_argument('--line-thickness', default=3, type=int, help='bounding box thickness (pixels)')
    parser.add_argument('--hide-labels', default=False, action='store_true', help='hide labels')
    parser.add_argument('--hide-conf', default=False, action='store_true', help='hide confidences')
    parser.add_argument('--half', action='store_true', help='use FP16 half-precision inference')
    parser.add_argument('--dnn', action='store_true', help='use OpenCV DNN for ONNX inference')
    # add qua
    parser.add_argument('--cfg', default='cpu', help='qua and off')
    parser.add_argument('--jit', type=bool,default=False)
    parser.add_argument('--offline_model', default='')
    opt = parser.parse_args()
    opt.imgsz *= 2 if len(opt.imgsz) == 1 else 1  # expand
    print_args(FILE.stem, opt)
    return opt


def main(opt):
    check_requirements(exclude=('tensorboard', 'thop'))
    run(**vars(opt))


if __name__ == "__main__":
    opt = parse_opt()
    main(opt)
